// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifndef BHV_BASIC_MOVE_H
#define BHV_BASIC_MOVE_H

#include <rcsc/geom/vector_2d.h>
#include <rcsc/player/soccer_action.h>
#include "domain_graph.h"
//#include "neck_turn_to_receiver.h"#
#include <rcsc/action/neck_scan_field.h>
#include "intention_receive.h"
#include "chain_action/action_chain_holder.h"

class Bhv_BasicMove
    : public rcsc::SoccerBehavior {
public:
    
    Bhv_BasicMove():
    	M_domain_graph(ActionChainHolder::i().domainGraph()),
    	M_predicate_complex(ActionChainHolder::i().predicateComplex())
    {  }
    
    bool execute( rcsc::PlayerAgent * agent );
    bool execute( rcsc::PlayerAgent * agent, DomainGraph::Ptr dgraph );
    
private:
    double getDashPower( const rcsc::PlayerAgent * agent );
    const DomainGraph &  M_domain_graph;
        PredicateComplex &  M_predicate_complex;
};

#endif


#ifndef BHV_BASIC_KEEP_OPEN_H
#define BHV_BASIC_KEEP_OPEN_H

#include <rcsc/geom/vector_2d.h>
#include <rcsc/player/soccer_action.h>
#include "domain_graph.h"
//#include "neck_turn_to_receiver.h"#
#include <rcsc/action/neck_scan_field.h>
#include "intention_receive.h"

class Bhv_BasicKeepOpen
: public rcsc::SoccerBehavior {
public:

    Bhv_BasicKeepOpen()
    {  }

    //bool execute( rcsc::PlayerAgent * agent );
    bool execute( rcsc::PlayerAgent* agent );

private:
    //double getDashPower( const rcsc::PlayerAgent * agent );

private:
    double evaluatePosition(const rcsc::WorldModel& wm, rcsc::Vector2D v_disp);

};

#endif


#include <rcsc/player/abstract_player_object.h>
//#include <rcsc/types.h>
//#include <functional>
#include <rcsc/common/server_param.h>
//#include <rcsc/player/player_object.h>


//
//#include <rcsc/player/localization.h>
//#include <rcsc/player/fullstate_sensor.h>

/*!
     \class PtrSelfDistCmp
     \brief predicate functor to compare player's distance from self
   */
   class PtrGoalDistCmp
   {
public:
       /*!
         \brief operation function
         \param lhs left hand side variable
         \param rhs right hand side variable
         \return compared result
        */
       bool operator()(const rcsc::AbstractPlayerObject* lhs,const rcsc::AbstractPlayerObject* rhs ) const
         {
             return (lhs->pos()).dist(rcsc::ServerParam::i().ourTeamGoalPos()) < (rhs->pos()).dist(rcsc::ServerParam::i().ourTeamGoalPos());
         }
   };



#include <rcsc/player/abstract_player_object.h>
//#include <rcsc/types.h>
//#include <functional>
#include <rcsc/common/server_param.h>
//#include <rcsc/player/player_object.h>


//
//#include <rcsc/player/localization.h>
//#include <rcsc/player/fullstate_sensor.h>

/*!
     \class PtrSelfDistCmp
     \brief predicate functor to compare player's distance from self
   */
   class PtrGoalBallDistCmp
   {
public:
       /*!
         \brief operation function
         \param lhs left hand side variable
         \param rhs right hand side variable
         \return compared result
        */
       bool operator()(const rcsc::AbstractPlayerObject* lhs,const rcsc::AbstractPlayerObject* rhs ) const
         {
             return (lhs->pos()).dist(rcsc::ServerParam::i().ourTeamGoalPos())+(lhs->distFromBall()) < (rhs->pos()).dist(rcsc::ServerParam::i().ourTeamGoalPos())+(rhs->distFromBall());
         }
   };
