// -*-c++-*-

/*!
  \file action_chain_holder.h
  \brief holder of calculated chain action Source File
*/

/*
 *Copyright:

 Copyright (C) Hiroki SHIMORA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 3, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "action_chain_holder.h"
#include <rcsc/player/world_model.h>

using namespace rcsc;

/*-------------------------------------------------------------------*/
/*!

 */
ActionChainHolder::ActionChainHolder()
    : M_graph(),
/////////////////////////////////////////////////////////////////////////////////////////////
      M_graph_2(),
/////////////////////////////////////////////////////////////////////////////////////////////
      M_evaluator(),
      M_generator(),
      M_domain_graph(),
      M_predicate_complex()
{

}

/*-------------------------------------------------------------------*/
/*!

 */
ActionChainHolder &
ActionChainHolder::instance()
{
    static ActionChainHolder s_instance;
    return s_instance;
}

/*-------------------------------------------------------------------*/
/*!

 */
const
ActionChainHolder &
ActionChainHolder::i()
{
    return instance();
}

/*-------------------------------------------------------------------*/
/*!

 */
void
ActionChainHolder::setFieldEvaluator( const FieldEvaluator::ConstPtr & evaluator )
{
    M_evaluator = evaluator;
}

/*-------------------------------------------------------------------*/
/*!

 */
void
ActionChainHolder::setActionGenerator( const ActionGenerator::ConstPtr & generator )
{
    M_generator = generator;
}
/*-------------------------------------------------------------------*/
/*!

 */
void
ActionChainHolder::setDomainGraph( const DomainGraph::Ptr dgraph)
{
    M_domain_graph = dgraph;
}

/*-------------------------------------------------------------------*/
/*!

 */
FieldEvaluator::ConstPtr
ActionChainHolder::fieldEvaluator() const
{
    return M_evaluator;
}

/*-------------------------------------------------------------------*/
/*!

 */
ActionGenerator::ConstPtr
ActionChainHolder::actionGenerator() const
{
    return M_generator;
}

/*-------------------------------------------------------------------*/
void
ActionChainHolder::setPredicateComplex( const PredicateComplex::Ptr pcomplex)
{
    M_predicate_complex=pcomplex;
}

/*!
 * 
 */

/*-------------------------------------------------------------------*/
/*!

 */
void
ActionChainHolder::update( const WorldModel & wm )
{
    static GameTime s_update_time( 0, 0 );
    static FieldEvaluator::ConstPtr s_update_evaluator;
    static ActionGenerator::ConstPtr s_update_generator;
    static DomainGraph::ConstPtr s_domain_graph;
    static PredicateComplex::ConstPtr s_predicate_complex;

    if ( s_update_time == wm.time()
            && s_update_evaluator == M_evaluator
            && s_update_generator == M_generator )
        //&& *s_domain_graph == M_domain_graph)
    {
        return;
    }
    s_update_time = wm.time();
    s_update_evaluator = M_evaluator;
    s_update_generator = M_generator;
    //s_domain_graph= &M_domain_graph;

    //generating a pass action to see who is the best candidate for a pass
    std::vector< ActionStatePair > candidatesP; //prepare the variables needed for generating actions
    const PredictState current_state( wm );
    const std::vector< ActionStatePair > empty_path;

    int toPassTo=0;
    CooperativeAction::ConstPtr pac=boost::shared_ptr<const CooperativeAction>();	//empty pointer
    //generate suitable actions for the a pass
    M_generator->generateFor(2, &candidatesP, current_state, wm, empty_path);


    if (candidatesP.size()>0)
    {
        pac=(CooperativeAction::ConstPtr)((*(candidatesP.begin())).actionPtr());
        toPassTo=(pac->targetPlayerUnum()!=wm.self().unum()?pac->targetPlayerUnum():0);
    }

    //find best link by optimising robustness and value
    M_domain_graph->findBestLink(wm,toPassTo);	



    if (M_domain_graph->getBestLink()!=-1)
    {


    	if (M_domain_graph->getBestLink()==2 && pac)	//use generated pass action
    		M_domain_graph->setAction(pac);
    	else		//generate a suitable action
    	{
    		std::vector< ActionStatePair > candidates; //prepare the variables needed for generating actions
    		const PredictState current_state( wm );
    		const std::vector< ActionStatePair > empty_path;

    		//generate suitable actions for the chosen link
    		M_generator->generateFor(M_domain_graph->getBestLink(), &candidates, current_state, wm, empty_path);


    		//M_domain_graph->setAction((*(candidates.begin())).actionPtr());
    		if (candidates.size()>0)
    		{
    			//std::cout<<"Candidates found: "<<(int)(  (*(candidates.begin())).action().category())<<std::endl;
    			dlog.addText( Logger::TEAM,"Candidates found: %d",(int)(  (*(candidates.begin())).action().category()  )  );

    			boost::shared_ptr<const CooperativeAction> ac=((*(candidates.begin())).actionPtr());
    			CooperativeAction::ConstPtr caac=(CooperativeAction::ConstPtr)ac;
    			M_domain_graph->setAction(caac);
    			dlog.addText( Logger::TEAM,"Action set: %d",(int)(caac->category()));
    		}
    		else
    		{
    			dlog.addText( Logger::TEAM,
    					"NO candidates found--link:%d",M_domain_graph->getBestLink());
    			M_domain_graph->setAction(boost::shared_ptr<const CooperativeAction>());
    		}
    	}
    }
    
    
    //if(M_domain_graph->active(wm.self().unum()) && M_domain_graph->action())
	//std::cout<<"setting action: "<<(int)(M_domain_graph->action()->category())<<std::endl;
    
    //M_graph =
    M_graph = 	ActionChainGraph::Ptr(new ActionChainGraph( M_evaluator, M_generator) );
    M_graph->calculate( wm );
    ////////////////////////////////////////////////////////////////////////////////////////////////
    //M_graph_2->calculate2( wm );
    ////////////////////////////////////////////////////////////////////////////////////////////////
}

/*-------------------------------------------------------------------*/
/*!

 */
const ActionChainGraph &
ActionChainHolder::graph() const
{
    return *M_graph;
}

/*-------------------------------------------------------------------*/
DomainGraph &
ActionChainHolder::domainGraph() const
{
    return *M_domain_graph;
}


PredicateComplex &
ActionChainHolder::predicateComplex() const
{
    return *M_predicate_complex;
}

/*-------------------------------------------------------------------*/
/*
 const DomainGraph &
ActionChainHolder::domainGraph() const
{
    return *M_domain_graph;
}

-------------------------------------------------------------------*/
